Omen =  AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceConsole-2.0")
local threat = AceLibrary("Threat-1.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local SML = AceLibrary("SharedMedia-1.0")
local roster = AceLibrary("Roster-2.1")

function Omen:OnInitialize()
	self:RegisterEvent("Threat_ThreatUpdated")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self.Anchor = CreateFrame("Frame", "OmenAnchor", UIParent)
	self.Anchor:SetWidth(200)
	self.Anchor:SetHeight(100)
	self.Anchor:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	self.Anchor:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = true,
		tileSize = 16,
		edgeSize = 16,
		insets = {left = 4, right = 4, top = 4, bottom = 4}
	})
	self.Anchor.title = self.Anchor:CreateFontString(nil, nil, "GameFontNormal")
	self.Anchor.title:SetPoint("BOTTOM", self.Anchor, "TOP", 0, 2)
	self.modName = "Omen 0.1"
	self.Anchor.title:SetText(self.modName)
	self.Anchor:SetBackdropColor(0, 0, 0, 0.5)
	self.Anchor:SetMovable(true)
	self.Anchor:EnableMouse(true)
	self.Anchor:SetScript("OnMouseDown", function() this:StartMoving(); end)
	self.Anchor:SetScript("OnMouseUp", function() this:StopMovingOrSizing(); end)
	self.bars = {}
	self.barList = {}
	self.barHeight = 15
	self.maxBars = 10
end

function Omen:PLAYER_TARGET_CHANGED()
	self.Anchor.title:SetText(string.format("%s - %s", self.modName, tostring(UnitName("target"))))
	self.threatTargetName = UnitName(Omen:GetThreatTarget())
	for k,v in pairs(self.bars) do
		local val = threat:GetThreat(k, self.threatTargetName) or 0
		v.threat:SetText(string.format("%2.0f", val))
		v.threatVal = tonumber(val)
	end
	Omen:ArrangeBars()
end

function Omen:Threat_ThreatUpdated(pName, pUnit, target, threatVal)
	if not threat:UnitIsUnit(self.threatTargetName, target) then return end
	if not self.bars[pName] then
		local _, cls = UnitClass(pUnit)
		local c = RAID_CLASS_COLORS[cls]
		self.bars[pName] = self:CreateBar()
		self.bars[pName].tex:SetVertexColor(c.r, c.g, c.b)
		tinsert(self.barList,self.bars[pName])
	end
	self.bars[pName].name:SetText(pName)
	self.bars[pName].threat:SetText(string.format("%2.0f", threatVal))
	self.bars[pName].threatVal = tonumber(threatVal)
	Omen:ArrangeBars()
end

function Omen:GetThreatTarget(root, depth)
	if depth == 5 then return "player" end
	local t = (root or "") .. "target"
	if UnitExists(t) then
		if UnitCanAttack("player", t) and not UnitPlayerControlled(t) then
			return t
		else
			return self:GetThreatTarget(t, (depth or 0) + 1)
		end
	end
	return "player"
end

function Omen:ArrangeBars()
	table.sort(self.barList, function(a,b)
		return b.threatVal < a.threatVal
	end)
	local lastBar = nil
	local maxThreat = -1
	local barCt = 0
	for k,v in pairs(self.barList) do 
		if maxThreat < v.threatVal then
			maxThreat = v.threatVal
			if maxThreat == 0 then maxThreat = 1 end
		end

		if v.threatVal > 0 then
			v:Show()
			v:ClearAllPoints()
			v:SetPercent(v.threatVal / maxThreat)
			if lastBar then
				v:SetPoint("TOPLEFT", lastBar, "BOTTOMLEFT", 0, 0)
				v:SetPoint("TOPRIGHT", lastBar, "BOTTOMRIGHT", 0, 0)
			else
				v:SetPoint("TOPLEFT", self.Anchor, "TOPLEFT", 5, -5)
				v:SetPoint("TOPRIGHT", self.Anchor, "TOPRIGHT", -5, -5)
			end
			lastBar = v
			barCt = barCt + 1
		else
			v:Hide()
		end
		if barCt >= self.maxBars then break end
	end
	self.Anchor:SetHeight(self.barHeight * max(1,#self.barList) + 10)
end

function Omen:CreateBar()
	local f = CreateFrame("Frame", "OmenBar" .. #self.bars, self.Anchor)
	f:SetHeight(self.barHeight)
	
	f.tex = f:CreateTexture()
	f.tex:SetTexture(SML:Fetch("statusbar","Glaze"))
	f.tex:SetPoint("TOPLEFT", f, "TOPLEFT", 0, 0)
	f.tex:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", 0, 0)
	f.tex:SetWidth(200)
	
	f.name 		= f:CreateFontString(nil, nil, "GameFontNormal")
	f.threat 	= f:CreateFontString(nil, nil, "GameFontNormal")
	f.name:SetVertexColor(1,1,1,1)
	f.threat:SetVertexColor(1,1,1,1)
	
	f.name:SetPoint("LEFT", f, "LEFT", 0, 0)
	f.threat:SetPoint("RIGHT", f, "RIGHT", 0, 0)
	f.SetPercent = function(self, pct) 
		local w = self:GetParent():GetWidth() - 10
		local width = w * pct
		if width == 0 then width = 1 end
		self.tex:SetWidth(width)
		end
	return f
end